@@ -160,24 +160,24 @@ function connect (packet, stream, opts) {
     }
 
     // Payload
+    length += 2 // payload length
     if (will.payload) {
       if (will.payload.length >= 0) {
         if (typeof will.payload === 'string') {
-          length += Buffer.byteLength(will.payload) + 2
+          length += Buffer.byteLength(will.payload)
         } else {
-          length += will.payload.length + 2
+          length += will.payload.length
         }
       } else {
         stream.emit('error', new Error('Invalid will payload'))
         return false
       }
-
-      // will properties
-      var willProperties = {}
-      if (protocolVersion === 5) {
-        willProperties = getProperties(stream, will.properties)
-        length += willProperties.length
-      }
+    }
+    // will properties
+    var willProperties = {}
+    if (protocolVersion === 5) {
+      willProperties = getProperties(stream, will.properties)
+      length += willProperties.length
     }
   }
 
@@ -794,7 +794,7 @@ function writeString (stream, string) {
   var strlen = Buffer.byteLength(string)
   writeNumber(stream, strlen)
 
-  stream.write(string, 'utf8')
+  return stream.write(string, 'utf8')
 }
 
 /**
@@ -859,9 +859,8 @@ function getProperties (stream, properties) {
     }
   }
   var propertiesLength = 0
-  function getLengthProperty (name) {
+  function getLengthProperty (name, value) {
     var type = protocol.propertiesTypes[name]
-    var value = properties[name]
     var length = 0
     switch (type) {
       case 'byte': {
@@ -926,7 +925,15 @@ function getProperties (stream, properties) {
           return false
         }
         length += Object.getOwnPropertyNames(value).reduce(function (result, name) {
-          result += 1 + 2 + Buffer.byteLength(name.toString()) + 2 + Buffer.byteLength(value[name].toString())
+          var currentValue = value[name]
+          if (Array.isArray(currentValue)) {
+            result += currentValue.reduce(function (currentLength, value) {
+              currentLength += 1 + 2 + Buffer.byteLength(name.toString()) + 2 + Buffer.byteLength(value.toString())
+              return currentLength
+            }, 0)
+          } else {
+            result += 1 + 2 + Buffer.byteLength(name.toString()) + 2 + Buffer.byteLength(value[name].toString())
+          }
           return result
         }, 0)
         break
@@ -940,7 +947,15 @@ function getProperties (stream, properties) {
   }
   if (properties) {
     for (var propName in properties) {
-      var propLength = getLengthProperty(propName)
+      var propLength = 0
+      var propValue = properties[propName]
+      if (Array.isArray(propValue)) {
+        for (var valueIndex = 0; valueIndex < propValue.length; valueIndex++) {
+          propLength += getLengthProperty(propName, propValue[valueIndex])
+        }
+      } else {
+        propLength = getLengthProperty(propName, propValue)
+      }
       if (!propLength) return false
       propertiesLength += propLength
     }
@@ -974,60 +989,78 @@ function getPropertiesByMaximumPacketSize (stream, properties, opts, length) {
   return propertiesData
 }
 
+function writeProperty (stream, propName, value) {
+  var type = protocol.propertiesTypes[propName]
+  switch (type) {
+    case 'byte': {
+      stream.write(Buffer.from([protocol.properties[propName]]))
+      stream.write(Buffer.from([+value]))
+      break
+    }
+    case 'int8': {
+      stream.write(Buffer.from([protocol.properties[propName]]))
+      stream.write(Buffer.from([value]))
+      break
+    }
+    case 'binary': {
+      stream.write(Buffer.from([protocol.properties[propName]]))
+      writeStringOrBuffer(stream, value)
+      break
+    }
+    case 'int16': {
+      stream.write(Buffer.from([protocol.properties[propName]]))
+      writeNumber(stream, value)
+      break
+    }
+    case 'int32': {
+      stream.write(Buffer.from([protocol.properties[propName]]))
+      write4ByteNumber(stream, value)
+      break
+    }
+    case 'var': {
+      stream.write(Buffer.from([protocol.properties[propName]]))
+      writeVarByteInt(stream, value)
+      break
+    }
+    case 'string': {
+      stream.write(Buffer.from([protocol.properties[propName]]))
+      writeString(stream, value)
+      break
+    }
+    case 'pair': {
+      Object.getOwnPropertyNames(value).forEach(function (name) {
+        var currentValue = value[name]
+        if (Array.isArray(currentValue)) {
+          currentValue.forEach(function (value) {
+            stream.write(Buffer.from([protocol.properties[propName]]))
+            writeStringPair(stream, name.toString(), value.toString())
+          })
+        } else {
+          stream.write(Buffer.from([protocol.properties[propName]]))
+          writeStringPair(stream, name.toString(), currentValue.toString())
+        }
+      })
+      break
+    }
+    default: {
+      stream.emit('error', new Error('Invalid property ' + propName + ' value: ' + value))
+      return false
+    }
+  }
+}
+
 function writeProperties (stream, properties, propertiesLength) {
   /* write properties to stream */
   writeVarByteInt(stream, propertiesLength)
   for (var propName in properties) {
     if (properties.hasOwnProperty(propName) && properties[propName] !== null) {
       var value = properties[propName]
-      var type = protocol.propertiesTypes[propName]
-      switch (type) {
-        case 'byte': {
-          stream.write(Buffer.from([protocol.properties[propName]]))
-          stream.write(Buffer.from([+value]))
-          break
-        }
-        case 'int8': {
-          stream.write(Buffer.from([protocol.properties[propName]]))
-          stream.write(Buffer.from([value]))
-          break
-        }
-        case 'binary': {
-          stream.write(Buffer.from([protocol.properties[propName]]))
-          writeStringOrBuffer(stream, value)
-          break
-        }
-        case 'int16': {
-          stream.write(Buffer.from([protocol.properties[propName]]))
-          writeNumber(stream, value)
-          break
-        }
-        case 'int32': {
-          stream.write(Buffer.from([protocol.properties[propName]]))
-          write4ByteNumber(stream, value)
-          break
-        }
-        case 'var': {
-          stream.write(Buffer.from([protocol.properties[propName]]))
-          writeVarByteInt(stream, value)
-          break
-        }
-        case 'string': {
-          stream.write(Buffer.from([protocol.properties[propName]]))
-          writeString(stream, value)
-          break
-        }
-        case 'pair': {
-          Object.getOwnPropertyNames(value).forEach(function (name) {
-            stream.write(Buffer.from([protocol.properties[propName]]))
-            writeStringPair(stream, name.toString(), value[name].toString())
-          })
-          break
-        }
-        default: {
-          stream.emit('error', new Error('Invalid property ' + propName))
-          return false
+      if (Array.isArray(value)) {
+        for (var valueIndex = 0; valueIndex < value.length; valueIndex++) {
+          writeProperty(stream, propName, value[valueIndex])
         }
+      } else {
+        writeProperty(stream, propName, value)
       }
     }
   }
